;**************************************************************************
;* FileName	: ADC2.AS
;* Date 	: 26.10.2010
;* File version : v1.00
								nolist
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Defines and macros for the A/D converter module
;*		  ===============================================
;*		  AD_INIT
;*		  AD_STARTCONV
;*		  AD_ENDCONV
;*		  AD_GET
;*		  AD_GET16
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* xtal_freq	EQU	4000000	  	; crystal frequency
;* core_freq	EQU	xtal_freq/4	; core operating frequency
;*
;*	include "..\modules\wait.mac"
;*	include "..\modules\wait.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define	DEBUG			; for some debug message
;*
;*-------------------------------------------------------------------------
;* Example: 
;*  		AD_INIT		AD_Fosc8, AD_JUST_LEFT, AD_REF_VSS_VDD, 00000000000001b
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  26/10/2010 	Initial Release (based on adc.mac 1.04)
;*			- support for newer ADs (PIC16F88x)
;*
;**************************************************************************

; check supported controllers
AD_PIC_SUPPORTED=0

	ifdef __16F882
AD_PIC_SUPPORTED=1
	endif

	ifdef __16F883
AD_PIC_SUPPORTED=1
	endif

	ifdef __16F884
AD_PIC_SUPPORTED=1
	endif

	ifdef __16F886
AD_PIC_SUPPORTED=1
	endif

	ifdef __16F887
AD_PIC_SUPPORTED=1
	endif

	if (AD_PIC_SUPPORTED==0)
		error "AD: this controller is not supported by ADC2 modul."
	endif


;--------------------------------------------------------------------------

			      ; ++++++++  - ADCON0 bits
#define		AD_Fosc2	00000001b
#define		AD_Fosc8	01000001b
#define		AD_Fosc32	10000001b
#define		AD_Frc		11000001b

;--------------------------------------------------------------------------

			      ; ++++++++  - ADCON0 bits
#define		AD_CH0		00000000b
#define		AD_CH1		00000100b
#define		AD_CH2		00001000b
#define		AD_CH3		00001100b
#define		AD_CH4		00010000b
#define		AD_CH5		00010100b
#define		AD_CH6		00011000b
#define		AD_CH7		00011100b
#define		AD_CH8		00100000b
#define		AD_CH9		00100100b
#define		AD_CH10		00101000b
#define		AD_CH11		00101100b
#define		AD_CH12		00110000b
#define		AD_CH13		00110100b
#define		AD_CH_CVREF	00111000b		; comparator reference
#define		AD_CH_FIXEDREF	00111100b		; fixed 0.6V reference

;--------------------------------------------------------------------------

			      ; ++++++++  - ADCON1 bits
#define		AD_JUST_LEFT	00000000b		; left justified
#define		AD_JUST_RIGHT	10000000b		; right justified

;--------------------------------------------------------------------------

			      ; ++++++++  - ADCON1 bits
#define		AD_REF_VSS_VDD	00000000b		; VSS   ... VDD 
#define		AD_REF_VSS_AN3	00010000b		; VSS   ... Vref+
#define		AD_REF_AN2_VDD	00100000b		; Vref- ... VDD 
#define		AD_REF_AN2_AN3	00110000b		; Vref- ... Vref+

;--------------------------------------------------------------------------




;***** AD_INIT macro ******************************************************
;* Initialize A/D converter 
;* Param : clksource	          		; A/D clock source (AD_Fxxx)
;*	   justify				; justify: AD_JUST_LEFT or AD_JUST_RIGHT
;*	   reference				; reference: AD_REF_xxx_yyy
;*	   portconfig				; port configurations bits 
;*						; bit 0-7  : ANS0-ANS7  from ANSEL
;*						; bit 8-13 : ANS8-ANS13 from ANSELH
;*						;            0=digital, 1=analog
;* Mem   : 12
;* Clock : 12  
;**************************************************************************
AD_INIT		macro	clksource, justify, reference, portconfig
		;==========================================================
									NOEXP
	variable AD_CLK  = (clksource)
	variable AD_TAD  = 0
	variable AD_TACQ = 20000			; 20 uS

	ifndef xtal_freq
	  error "AD_INIT: xtal_freq was not definied!"
	  exitm
	endif

	if (AD_CLK==AD_Fosc2)
	  variable AD_TAD = 1000000000/(xtal_freq)*2
	endif

	if (AD_CLK==AD_Fosc8)
	  variable AD_TAD = 1000000000/(xtal_freq)*8
	endif

	if (AD_CLK==AD_Fosc32)
	  variable AD_TAD = 1000000000/(xtal_freq)*32
	endif

	if (AD_CLK==AD_Frc)
	  variable AD_TAD = 6000			; 6000 nS (max.)
	endif

	#ifdef DEBUG
	  messg AD_INIT: AD_TAD <#v(AD_TAD)> nS
	#endif
 

	if (AD_TAD==0)
	  error AD_INIT: Value of clksource is wrong.
	endif

	if (AD_TAD<1600)
	  error AD_INIT: Value of AD_TAD violate the minimum required Tad time. 
	endif

	if (AD_TAD>20000)
	  messg AD_INIT: For faster conversion selection of another clock source is recommended.
	endif

	#define INC_ADC
									EXP
			banksel	ANSEL
			ERRORLEVEL -302
			movlw	low portconfig
			movwf	ANSEL
			movlw	high portconfig
			movwf	ANSELH
			banksel	ADCON1
			movlw	(justify) | (reference)
			movwf	ADCON1
			ERRORLEVEL +302
			BANK0


		endm	


;***** AD_STARTCONV macro *************************************************
;* start an AD conversion on a channel
;* Param : -
;* Input : W <- channel          		; AD_CH0 ... AD_CH13
;* Mem   : 1
;* Clock : ?
;**************************************************************************
AD_STARTCONV	macro
		;==========================================================
			call	AD_startconv

		
		endm



;***** AD_ENDCONV macro ***************************************************
;* wait end of conversion
;* Param : -
;* Mem   : 1
;* Clock : ?
;**************************************************************************
AD_ENDCONV	macro
		;==========================================================
			call	AD_endconv

		
		endm



;***** AD_GET macro *******************************************************
;* get a converted value
;* Param : channel	          		; AD_CH0 ... AD_CH13
;* Output: ADRESH:ADRESL <- result  		; result
;* Mem   : 3
;* Clock : TACQ + 10*TAD
;**************************************************************************
AD_GET		macro	channel
		;==========================================================
			movlw	channel
			call	AD_startconv
			call	AD_endconv


		endm



;***** AD_GET16 macro *****************************************************
;* get a converted value
;* Param : channel	          		; AD_CH0 ... AD_CH13
;* Output: reg16 <- result         		; result
;* Mem   : 3
;* Clock : TACQ + 10*TAD
;**************************************************************************
AD_GET16	macro	channel, reg16
		;==========================================================
			movlw	channel
			call	AD_startconv
			call	AD_endconv
			bank1
			movf	LL ADRESL, w
			bank0 
			movwf	reg16+1
			movf	ADRESH, w
			movwf	reg16
			

		endm


;**************************************************************************
		list

